﻿/* このフォームはスクリーン全体のイメージをコピーしてマウスによるクリック位置の色を抽出するために使う
   ただし、WPF には Graphics クラスの CopyFromScreen メソッドに相当する機能がないので、
   System.Windows.Forms.Form を使うことにした。
   フォームのデザイナは必要ないので、コードだけを使ってフォームを作成する。
*/

using System;
using System.Collections.Generic;
using System.Drawing;
using System.Windows.Forms;

namespace ColorPicker
{
	internal class Capture : System.Windows.Forms.Form
	{
		private Bitmap FBitmap = null;
		private Color FColor;
		private int FX = 0;
		private int FY = 0;

		// プロパティ
		public Color Color { get { return FColor; } }

		public Capture()
		{
			FBitmap = new Bitmap(Screen.PrimaryScreen.Bounds.Width, Screen.PrimaryScreen.Bounds.Height);
			Graphics g = Graphics.FromImage(FBitmap);

			g.CopyFromScreen(new Point(0, 0), new Point(0, 0),
				new Size(Screen.PrimaryScreen.Bounds.Width, Screen.PrimaryScreen.Bounds.Height));

			g.Dispose();

			this.FormBorderStyle = FormBorderStyle.None;
			this.SetBounds(0, 0, Screen.PrimaryScreen.Bounds.Width, Screen.PrimaryScreen.Bounds.Height);
			this.TopMost = true;
		}

		protected override void OnClick(EventArgs e)
		{
			this.Close();
		}

		protected override void OnPaint(PaintEventArgs e)
		{
			if (FBitmap != null)
			{
				e.Graphics.DrawImage(FBitmap, 0, 0);
			}
		}

		protected override void OnFormClosing(FormClosingEventArgs e)
		{
			FColor = FBitmap.GetPixel(FX, FY);

			if (FBitmap != null)
				FBitmap.Dispose();
		}

		protected override void OnMouseDown(MouseEventArgs e)
		{
			FX = e.X;
			FY = e.Y;
		}

	} // end of Capture class
} // end of namespace
