﻿/*	WPF 対応カラー選択コンボボックス

		WPF では標準のコントロールの種類が少ないが、自作することが容易なので、それほど困るわけではない。
		カラー選択する機会は多いので、カラー選択コンボボックスを作ってみた。

	【使い方】
		基本的には通常のコンボボックスと同じ手順で使える。
		プロパティは SolidColorBrush オブジェクトに対する Brush プロパティしかない。
		もし、SolidColorBrush オブジェクトではなく、カラーがほしければ、ブラシの Color プロパティを参照すればよい。

	【著作権】
		本クラスはフリーウエアである。コードの改変・流用は無制限に許可する。ただし、このクラスを使ったことによる
		すべての不都合に対して著作権者は免責されるものとする。

		2010/03/09
		佐藤 正
		e-mail   : pmansato@kanazawa-net.ne.jp
	  HomePage : http://www.kanazawa-net.ne.jp/~pmansato/

*/
using System;
using System.Collections.Generic;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Media;
using System.Reflection; // BindingFlags

namespace emanual.Wpf.Control
{
	public class ColorComboBox : System.Windows.Controls.ComboBox
	{
		private SolidColorBrush FBrush = Brushes.AliceBlue;

		// プロパティ
		public SolidColorBrush Brush
		{
			get { return FBrush; }
			set
			{
				FBrush = value;
				this.SetItem(FBrush);
			} 
		}

		//-------------------------------------------------------------------------------------
		// コンストラクタ
		public ColorComboBox()
		{
			this.SetupControl();
		}

		//-------------------------------------------------------------------------------------
		// Brush プロパティを設定したときにその項目を選択する
		private void SetItem(SolidColorBrush brush)
		{
			int index = 0;

			for (int i = 0; i < this.Items.Count; i++)
			{
				ComboBoxItem item = this.Items[i] as ComboBoxItem;
				SolidColorBrush b = item.Tag as SolidColorBrush;

				if (b == brush)
				{
					index = i;
					break;
				}
			}

			this.SelectedIndex = index;
		}

		//-------------------------------------------------------------------------------------
		// コントロールのセットアップ
		private void SetupControl()
		{
			Type type = typeof(System.Windows.Media.Brushes);
			BindingFlags flags = BindingFlags.Static | BindingFlags.Public;

			PropertyInfo[] infos = type.GetProperties(flags);

			foreach (PropertyInfo info in infos)
			{
				SolidColorBrush brush = info.GetValue(null, null) as SolidColorBrush;

				ComboBoxItem item = new ComboBoxItem();

				StackPanel panel = new StackPanel();
				panel.Orientation = Orientation.Horizontal;

				Border border = new Border();
				border.Background = brush;
				border.CornerRadius = new CornerRadius(3);
				border.Width = 30;
				border.Height = 12;
				border.BorderBrush = Brushes.Black;
				border.BorderThickness = new Thickness(1);

				TextBlock block = new TextBlock();
				block.Text = info.Name;
				block.Width = this.Width - border.Width - 20;
				block.Margin = new Thickness(10, 0, 0, 0);

				panel.Children.Add(border);
				panel.Children.Add(block);

				item.Content = panel;
				item.Tag = brush;

				this.Items.Add(item);
			}

			if (this.Items.Count > 0)
				this.SelectedIndex = 0;
		}

		//-----------------------------------------------------------------------------------
		// 項目を選択したときに Brush プロパティを設定する
		protected override void OnSelectionChanged(SelectionChangedEventArgs e)
		{
			ComboBoxItem item = this.SelectedItem as ComboBoxItem;
			FBrush = item.Tag as SolidColorBrush;
			base.OnSelectionChanged(e);
		}

		//-------------------------------------------------------------------------------------
	} // end of ColorComboBox class
} // end of namespace
