﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data; // IMultiValueConverter
using System.Windows.Input;
using System.Windows.Media;
using Microsoft.Windows.Shell; // WindowChrome
using System.Diagnostics; // Debug

namespace ColorPicker
{
	public partial class frmColorPicker : Window
	{
		public frmColorPicker()
		{
			InitializeComponent();
		}

		//-------------------------------------------------------------------------------------
		private void Window_Loaded(object sender, RoutedEventArgs e)
		{
			this.SetRectColor(Colors.White);
		}

		//-------------------------------------------------------------------------------------
		private void closeCommand_Executed(object sender, ExecutedRoutedEventArgs e)
		{
			SystemCommands.CloseWindow((Window)e.Parameter);
		}

		//-------------------------------------------------------------------------------------
		private void minimizeWindow_Executed(object sender, ExecutedRoutedEventArgs e)
		{
			SystemCommands.MinimizeWindow((Window)e.Parameter);
		}

		//-------------------------------------------------------------------------------------
		private void btnColorValue_Click(object sender, RoutedEventArgs e)
		{
			Color color = this.GetColorFromString(txtColorValue.Text);
			this.SetRectColor(color);

			Clipboard.SetText(txtColorValue.Text);
		}

		//-------------------------------------------------------------------------------------
		private void txtColorValue_KeyDown(object sender, KeyEventArgs e)
		{
			// 入力後、[Return] を押したとき
			if (e.Key == Key.Return)
			{
				this.btnColorValue_Click(sender, null);
			}
		}

		//-------------------------------------------------------------------------------------
		private void ColorComboBox_SelectionChanged(object sender, SelectionChangedEventArgs e)
		{
			this.SetRectColor(cmbColor.Brush.Color);
		}

		//-------------------------------------------------------------------------------------
		private void btnColorPicker_Click(object sender, RoutedEventArgs e)
		{
			Capture form = new Capture();
			form.ShowDialog();

			Color color = Color.FromRgb(form.Color.R, form.Color.G, form.Color.B);

			form.Dispose();

			this.SetRectColor(color);
		}

		//-------------------------------------------------------------------------------------
		// HSL 用スライダーを操作したときの処理
		// ARGB 用スライダーほうはバインディングで処理
		private void slider_ValueChanged(object sender, RoutedPropertyChangedEventArgs<double> e)
		{
			Slider slider = sender as Slider;

			if (slider != null)
			{
				try
				{
					Color color = frmColorPicker.HslToRgb(sliderHue.Value, sliderSaturation.Value, sliderLightness.Value);

					sliderRed.Value = color.R;
					sliderGreen.Value = color.G;
					sliderBlue.Value = color.B;

					rectColor.Fill = new SolidColorBrush(color);
				}
				catch
				{
				}
			}
		}

		//-------------------------------------------------------------------------------------
		private void imgIcon_MouseLeftButtonDown(object sender, MouseButtonEventArgs e)
		{
			frmAbout form = new frmAbout();
			form.ShowDialog();
		}

		//-------------------------------------------------------------------------------------
		// rectColor の Background プロパティを設定し、各スライダーの Value プロパティを設定する
		// ARGB 用スライダーによる操作はバインディングを使っているので、XAML コードを参照のこと
		private void SetRectColor(Color color)
		{
			// このプロパティを変更すると、txtColorValue の内容はバインディングを使って変更する
			rectColor.Fill = new SolidColorBrush(color);

			sliderAlpha.Value = color.A;
			sliderRed.Value = color.R;
			sliderGreen.Value = color.G;
			sliderBlue.Value = color.B;

			double hue = 0, saturation = 0, lightness = 0;
			frmColorPicker.RgbToHsl(color, ref hue, ref saturation, ref lightness);

			sliderHue.Value = (int)hue;
			sliderSaturation.Value = (float)saturation;
			sliderLightness.Value = (float)lightness;
		}

		#region private methods
		//-------------------------------------------------------------------------------------
		// txtColorValue に入力したテキストを Color 型に変換する
		private Color GetColorFromString(string s)
		{
			Color color = Colors.White;
			System.ComponentModel.TypeConverter converter = System.ComponentModel.TypeDescriptor.GetConverter(typeof(Color));

			try
			{
				color = (Color)converter.ConvertFromString(s);
			}
			catch
			{
				MessageBox.Show("入力データが不正です。\n例にならってください。");
			}

			return color;
		}

		//-------------------------------------------------------------------------------------
		// Color 型から HSL 値に変換する
		static public void RgbToHsl(Color color, ref double hue, ref double saturation, ref double lightness)
		{
			double r, g, b, h, s, l;

			r = color.R / 255.0;
			g = color.G / 255.0;
			b = color.B / 255.0;

			double maxColor = Math.Max(r, Math.Max(g, b));
			double minColor = Math.Min(r, Math.Min(g, b));

			if (r == g && r == b)
			{
				h = 0.0;
				s = 0.0;
				l = r;
			}
			else
			{
				l = (minColor + maxColor) / 2;
				if (l < 0.5)
					s = (maxColor - minColor) / (maxColor + minColor);
				else
					s = (maxColor - minColor) / (2.0 - maxColor - minColor);

				if (r == maxColor)
					h = (g - b) / (maxColor - minColor);
				else if (g == maxColor)
					h = 2.0 + (b - r) / (maxColor - minColor);
				else
					h = 4.0 + (r - g) / (maxColor - minColor);

				h /= 6;

				if (h < 0)
					++h;
			}

			// 0 ～ 1 の範囲内に制限する
			if (h < 0) h = 0;
			if (h > 1) h = 1;
			if (s < 0) s = 0;
			if (s > 1) s = 1;
			if (l < 0) l = 0;
			if (l > 1) l = 1;

			hue = h * 360;
			saturation = s;
			lightness = l;
		}

		//-------------------------------------------------------------------------------------
		// HSL 値から Color 型に変換する
		// hue        : 0 ～ 360
		// saturation : 0 ～ 1
		// lightness  : 0 ～ 1
		public static Color HslToRgb(double hue, double saturation, double lightness)
		{
			double r, g, b, h, s, l, s1, s2, r1, g1, b1;

			h = hue / 360.0;
			s = saturation;
			l = lightness;

			if (s == 0)
			{
				r = g = b = l;
			}
			else
			{
				if (l < 0.5)
				{
					s2 = l * (1 + s);
				}
				else
				{
					s2 = (l + s) - (l * s);
				}

				s1 = 2 * l - s2;
				r1 = h + 1.0 / 3.0;

				if (r1 > 1)
				{
					--r1;
				}

				g1 = h;
				b1 = h - 1.0 / 3.0;

				if (b1 < 0)
					++b1;

				// R
				if (r1 < 1.0 / 6.0)
					r = s1 + (s2 - s1) * 6.0 * r1;
				else if (r1 < 0.5)
					r = s2;
				else if (r1 < 2.0 / 3.0)
					r = s1 + (s2 - s1) * ((2.0 / 3.0) - r1) * 6.0;
				else
					r = s1;

				// G
				if (g1 < 1.0 / 6.0)
					g = s1 + (s2 - s1) * 6.0 * g1;
				else if (g1 < 0.5)
					g = s2;
				else if (g1 < 2.0 / 3.0)
					g = s1 + (s2 - s1) * ((2.0 / 3.0) - g1) * 6.0;
				else g = s1;

				// B
				if (b1 < 1.0 / 6.0)
					b = s1 + (s2 - s1) * 6.0 * b1;
				else if (b1 < 0.5)
					b = s2;
				else if (b1 < 2.0 / 3.0)
					b = s1 + (s2 - s1) * ((2.0 / 3.0) - b1) * 6.0;
				else
					b = s1;
			}

			// 0 ～ 1 の範囲内に制限する
			if (h < 0) h = 0;
			if (h > 1) h = 1;
			if (s < 0) s = 0;
			if (s > 1) s = 1;
			if (l < 0) l = 0;
			if (l > 1) l = 1;

			return Color.FromRgb(Convert.ToByte(r * 255), Convert.ToByte(g * 255), Convert.ToByte(b * 255));
		}

		#endregion

	} // end of frmColorPicker

	//**************************************************************************************
	// Converter class
	//**************************************************************************************
	// ARGB および HSL 値から Color 型に変換する
	public class PickColorConverter : IMultiValueConverter
	{
		// スライダーを操作したときに呼び出されるメソッドで、戻り値は rectColor の Background を設定する
		public object Convert(object[] values, Type targetType, object parameter, System.Globalization.CultureInfo culture)
		{
			byte A = (byte)(double)(values[0]);
			byte R = (byte)(double)(values[1]);
			byte G = (byte)(double)(values[2]);
			byte B = (byte)(double)(values[3]);

			return new SolidColorBrush(Color.FromArgb(A, R, G, B));
		}

		//-----------------------------------------------------------------------------------------------
		// rectColor の Background を変更したときに呼び出されるメソッドで、各スライダーの Value プロパティを設定する
		public object[] ConvertBack(object value, Type[] targetTypes, object parameter, System.Globalization.CultureInfo culture)
		{
			SolidColorBrush brush = (SolidColorBrush)value;
			Color color = brush.Color;

			object[] result = new Object[] { (double)color.A, (double)color.R, (double)color.G, (double)color.B };

			return result;
		}
		//-------------------------------------------------------------------------------------
	} // end of PickColorConverter class

	//**************************************************************************************
	// sliderAlpha, Red, Green, Blue の Value プロパティを書式化する
	public class ArgbValueConverter : IValueConverter
	{
		public object Convert(object value, Type targetType, object parameter, System.Globalization.CultureInfo culture)
		{
			string result = "";

			int sliderValue = (int)(double)value;

			result = String.Format("{0,3:D}", sliderValue);

			return result;
		}

		//-------------------------------------------------------------------------------------
		public object ConvertBack(object value, Type targetType, object parameter, System.Globalization.CultureInfo culture)
		{
			return Binding.DoNothing;
		}
	} // end of HueValueConverter class

	//**************************************************************************************
	// sliderHue の Value プロパティを書式化する
	public class SliderHueValueConverter : IValueConverter
	{
		public object Convert(object value, Type targetType, object parameter, System.Globalization.CultureInfo culture)
		{
			string result = "";

			int sliderValue = (int)(double)value;

			result = String.Format("{0,3:D}", sliderValue);

			return result;
		}

		//-------------------------------------------------------------------------------------
		public object ConvertBack(object value, Type targetType, object parameter, System.Globalization.CultureInfo culture)
		{
			return Binding.DoNothing;
		}
	} // end of HueValueConverter class

	//**************************************************************************************
	// sliderSaturation および sliderLightness の Value プロパティを書式化する
	public class SliderSaturationValueConverter : IValueConverter
	{
		public object Convert(object value, Type targetType, object parameter, System.Globalization.CultureInfo culture)
		{
			string result = "";

			double sliderValue = (double)value;

			result = String.Format("{0,3:F}", sliderValue);

			return result;
		}

		//-------------------------------------------------------------------------------------
		public object ConvertBack(object value, Type targetType, object parameter, System.Globalization.CultureInfo culture)
		{
			return Binding.DoNothing;
		}
	} // end of SliderSaturationValueConverter class

	//**************************************************************************************
	// SolidColorBrush を Color 値に変換する
	public class BrushToColorValueConverter : IValueConverter
	{
		public object Convert(object value, Type targetType, object parameter, System.Globalization.CultureInfo culture)
		{
			string result = "";

			SolidColorBrush brush = (SolidColorBrush)value;
			
			if (brush != null)
				result = String.Format("{0}", brush.Color);

			return result;
		}

		//-------------------------------------------------------------------------------------
		public object ConvertBack(object value, Type targetType, object parameter, System.Globalization.CultureInfo culture)
		{
			return Binding.DoNothing;
		}
	} // end of BrushToColorValueConverter class

	//**************************************************************************************
	// 整数を 16 進数に変換する
	[ValueConversion(typeof(double), typeof(int))]
	public class IntToHexValueConverter : IValueConverter
	{
		public object Convert(object value, Type targetType, object parameter, System.Globalization.CultureInfo culture)
		{
			double textValue = (double)value;

			return String.Format("(0x{0:X2})", (int)textValue);
		}

		//-------------------------------------------------------------------------------------
		public object ConvertBack(object value, Type targetType, object parameter, System.Globalization.CultureInfo culture)
		{
			return Binding.DoNothing;
		}
	} // end of IntToHexValueConverter class

} // end of namespace
